<?php
/**
 * "WooCommerce Product" ACFE Form Action Class.
 *
 * Handles the "WooCommerce Product" ACFE Form Action.
 *
 * @package Conditional_Form_Actions_For_ACFE
 */

// Exit if accessed directly.
defined( 'ABSPATH' ) || exit;

/**
 * "WooCommerce Product" ACFE Form Action Class.
 *
 * A class that handles the "WooCommerce Product" ACFE Form Action.
 *
 * @since 0.2.0
 */
class CFAFA_ACFE_Form_Action_Product extends CFAFA_ACFE_Form_Action_Base {

	/**
	 * Plugin object.
	 *
	 * @since 0.2.0
	 * @access public
	 * @var Conditional_Form_Actions_For_ACFE
	 */
	public $plugin;

	/**
	 * ACF Extended object.
	 *
	 * @since 0.2.0
	 * @access public
	 * @var CFAFA_ACFE
	 */
	public $acfe;

	/**
	 * Form Action Name.
	 *
	 * @since 0.2.0
	 * @access public
	 * @var string
	 */
	public $name = 'woo_cfafa_product';

	/**
	 * Form Action Name.
	 *
	 * @since 0.2.0
	 * @access public
	 * @var string
	 */
	public $action_name = 'woo_cfafa_product';

	/**
	 * Field Key Prefix.
	 *
	 * @since 0.2.0
	 * @access public
	 * @var string
	 */
	public $field_key = 'field_';

	/**
	 * Field Name Prefix.
	 *
	 * @since 0.2.0
	 * @access public
	 * @var string
	 */
	public $field_name = '';

	/**
	 * Constructor.
	 *
	 * @since 0.2.0
	 */
	public function __construct() {

		// Store references to objects.
		$this->plugin = cfafa();
		$this->acfe   = $this->plugin->acfe;

		// Label this Form Action.
		$this->title = __( 'WooCommerce Product action', 'conditional-form-actions-for-acfe' );

		// Alias Placeholder for this Form Action.
		$this->alias_placeholder = __( 'WooCommerce Product', 'conditional-form-actions-for-acfe' );

		// Declare core Fields for this Form Action.
		$this->item = [
			'action'     => $this->name,
			'name'       => '',
			'product_id' => false,
		];

		// Init parent.
		parent::__construct();

	}

	/**
	 * Prepares data for saving when the Form Action is saved.
	 *
	 * @since 0.2.0
	 *
	 * @param array $action The array of Form Action data.
	 * @return array $save The array of data to save.
	 */
	public function prepare_save_action( $action ) {

		// Init with default array for this Field.
		$save = $this->item;

		// Always add action name.
		$save['name'] = $action['name'];

		// Always save Conditional Field.
		if ( acf_maybe_get( $action, $this->conditional_code ) ) {
			$save['conditional'] = $action[ $this->conditional_code ];
		}

		// Product data.
		if ( acf_maybe_get( $action, 'product_id' ) ) {
			$save['product_id'] = $action['product_id'];
		}

		// --<
		return $save;

	}

	/**
	 * Performs the action when the Form the Action is attached to is submitted.
	 *
	 * @since 0.2.0
	 *
	 * @param array  $form The array of Form data.
	 * @param string $action The customised name of the action.
	 */
	public function make_action( $form, $action ) {

		// Bail if a filter has overridden the action.
		if ( false === $this->make_skip( $form, $action ) ) {
			return;
		}

		// Populate Conditional Reference and value.
		$action = $this->form_conditional_populate( $action );

		// Populate Product data array.
		$product = $this->form_product_data( $form, $action );

		// Check Conditional.
		if ( $this->form_conditional_check( $action ) ) {

			// Act with the data from the Form.
			$product = $this->form_product_save( $product );

		}

		// Build result.
		$result = [
			'product'     => $product,
			'conditional' => $action['conditional'],
		];

		// Save the results of this Action for later use.
		$this->make_action_save( $action, $result );

	}

	/**
	 * Defines additional Fields for the "Action" Tab.
	 *
	 * @since 0.2.0
	 *
	 * @return array $fields The array of Fields for this section.
	 */
	protected function tab_action_append() {

		// Init Fields.
		$fields = [];

		$fields[] = [
			'key'               => $this->field_key . 'product_id',
			'label'             => __( 'WooCommerce Product', 'conditional-form-actions-for-acfe' ),
			'name'              => 'product_id',
			'type'              => 'select',
			'instructions'      => __( 'Use this to add a WooCommerce Product to the Cart.', 'conditional-form-actions-for-acfe' ),
			'required'          => 0,
			'conditional_logic' => 0,
			'wrapper'           => [
				'width'                      => '',
				'class'                      => '',
				'id'                         => '',
				'data-instruction-placement' => 'field',
			],
			'acfe_permissions'  => '',
			'default_value'     => '',
			'placeholder'       => '',
			'allow_null'        => 1,
			'multiple'          => 0,
			'ui'                => 0,
			// 'ajax' => 1,
			// 'ajax_action' => 'cfafa_get_products',
			'return_format'     => 'value',
			'choices'           => $this->product_choices_get(),
		];

		// Configure Conditional Field.
		$args = [
			'placeholder'  => __( 'Always add', 'conditional-form-actions-for-acfe' ),
			'instructions' => __( 'To add the Product to the Cart only when a Form Field is populated (e.g. "First Name") link this to the Form Field. To add the Product to the Cart only when more complex conditions are met, link this to a Hidden Field with value "1" where the conditional logic of that Field shows it when the conditions are met.', 'conditional-form-actions-for-acfe' ),
		];

		// Add Conditional Field.
		$fields[] = $this->form_conditional_field_get( $args );

		// --<
		return $fields;

	}

	/**
	 * Builds Product data array from mapped Fields.
	 *
	 * @since 0.2.0
	 *
	 * @param array $form The array of Form data.
	 * @param array $action The array of Action data.
	 * @return array $data The array of results of this Action to save for later use.
	 */
	private function form_product_data( $form, $action ) {

		// Init data array.
		$data = [];

		// Set ACFE "context". We want unformatted data.
		$context = [
			'context' => 'save',
			'format'  => false,
		];

		// Get Product ID.
		acfe_apply_tags( $action['product_id'], $context );
		$product_id = $action['product_id'];

		// Add Product ID.
		$data['product_id'] = $product_id;

		// --<
		return $data;

	}

	/**
	 * Sends the WooCommerce Product given data from mapped Fields.
	 *
	 * @since 0.2.0
	 *
	 * @param array $product_data The array of Product data.
	 * @return array|bool $product_data The Product data array, or false on failure.
	 */
	private function form_product_save( $product_data ) {

		// Strip out empty Fields.
		$product_data = $this->form_data_prepare( $product_data );
		if ( empty( $product_data['product_id'] ) ) {
			return false;
		}

		// Add the Product to the WooCommerce Cart.
		$result = $this->product_add_to_cart( $product_data );
		if ( false === $result ) {
			return false;
		}

		// Add Cart Item key to Product data.
		$product_data['cart_item_key'] = $result;

		// --<
		return $product_data;

	}

	/**
	 * Gets the WooCommerce Products as an array of options for ACF.
	 *
	 * @since 0.2.0
	 *
	 * @return array $options The array of Products formatted for ACF.
	 */
	private function product_choices_get() {

		// Init return.
		$options = [];

		// We need all Products.
		$args = [
			'limit' => -1,
		];

		// Query Products.
		$products = wc_get_products( $args );

		// Build options for ACF select.
		if ( $products ) {
			foreach ( $products as $product ) {
				$options[ (int) $product->get_id() ] = $product->get_name();
			}
		}

		// --<
		return $options;

	}

	/**
	 * Adds the Product to the Cart.
	 *
	 * @since 0.2.0
	 *
	 * @param array $product_data The array of Product data.
	 * @return integer|bool $cart_item_key The key of the Cart Item, or false otherwise.
	 */
	private function product_add_to_cart( $product_data ) {

		// Get the Product.
		$product = wc_get_product( $product_data['product_id'] );
		if ( empty( $product ) ) {
			return false;
		}

		// Some defaults.
		$product_id     = $product->get_id();
		$quantity       = 1;
		$variation_id   = 0;
		$variation      = [];
		$cart_item_data = [];

		// Build Cart Item data?

		// Add the configured Product to the Cart.
		$cart_item_key = WC()->cart->add_to_cart( $product_id, $quantity, $variation_id, $variation, $cart_item_data );

		// --<
		return $cart_item_key;

	}

}
